@@ -9,11 +9,14 @@ from tianshou.env.worker import (
     RayEnvWorker,
     SubprocEnvWorker,
 )
-from tianshou.utils import RunningMeanStd
+
+GYM_RESERVED_KEYS = [
+    "metadata", "reward_range", "spec", "action_space", "observation_space"
+]
 
 
-class BaseVectorEnv(gym.Env):
-    """Base class for vectorized environments wrapper.
+class BaseVectorEnv(object):
+    """Base class for vectorized environments.
 
     Usage:
     ::
@@ -61,13 +64,6 @@ class BaseVectorEnv(gym.Env):
     :param float timeout: use in asynchronous simulation same as above, in each
         vectorized step it only deal with those environments spending time
         within ``timeout`` seconds.
-    :param bool norm_obs: Whether to track mean/std of data and normalize observation
-        on return. For now, observation normalization only support observation of
-        type np.ndarray.
-    :param obs_rms: class to track mean&std of observation. If not given, it will
-        initialize a new one. Usually in envs that is used to evaluate algorithm,
-        obs_rms should be passed in. Default to None.
-    :param bool update_obs_rms: Whether to update obs_rms. Default to True.
     """
 
     def __init__(
@@ -76,9 +72,6 @@ class BaseVectorEnv(gym.Env):
         worker_fn: Callable[[Callable[[], gym.Env]], EnvWorker],
         wait_num: Optional[int] = None,
         timeout: Optional[float] = None,
-        norm_obs: bool = False,
-        obs_rms: Optional[RunningMeanStd] = None,
-        update_obs_rms: bool = True,
     ) -> None:
         self._env_fns = env_fns
         # A VectorEnv contains a pool of EnvWorkers, which corresponds to
@@ -106,12 +99,6 @@ class BaseVectorEnv(gym.Env):
         self.ready_id = list(range(self.env_num))
         self.is_closed = False
 
-        # initialize observation running mean/std
-        self.norm_obs = norm_obs
-        self.update_obs_rms = update_obs_rms
-        self.obs_rms = RunningMeanStd() if obs_rms is None and norm_obs else obs_rms
-        self.__eps = np.finfo(np.float32).eps.item()
-
     def _assert_is_not_closed(self) -> None:
         assert not self.is_closed, \
             f"Methods of {self.__class__.__name__} cannot be called after close."
@@ -127,9 +114,7 @@ class BaseVectorEnv(gym.Env):
         ``action_space``. However, we would like the attribute lookup to go straight
         into the worker (in fact, this vector env's action_space is always None).
         """
-        if key in [
-            'metadata', 'reward_range', 'spec', 'action_space', 'observation_space'
-        ]:  # reserved keys in gym.Env
+        if key in GYM_RESERVED_KEYS:  # reserved keys in gym.Env
             return self.get_env_attr(key)
         else:
             return super().__getattribute__(key)
@@ -137,7 +122,7 @@ class BaseVectorEnv(gym.Env):
     def get_env_attr(
         self,
         key: str,
-        id: Optional[Union[int, List[int], np.ndarray]] = None
+        id: Optional[Union[int, List[int], np.ndarray]] = None,
     ) -> List[Any]:
         """Get an attribute from the underlying environments.
 
@@ -162,7 +147,7 @@ class BaseVectorEnv(gym.Env):
         self,
         key: str,
         value: Any,
-        id: Optional[Union[int, List[int], np.ndarray]] = None
+        id: Optional[Union[int, List[int], np.ndarray]] = None,
     ) -> None:
         """Set an attribute in the underlying environments.
 
@@ -183,7 +168,7 @@ class BaseVectorEnv(gym.Env):
 
     def _wrap_id(
         self,
-        id: Optional[Union[int, List[int], np.ndarray]] = None
+        id: Optional[Union[int, List[int], np.ndarray]] = None,
     ) -> Union[List[int], np.ndarray]:
         if id is None:
             return list(range(self.env_num))
@@ -197,8 +182,10 @@ class BaseVectorEnv(gym.Env):
                 f"Can only interact with ready environments {self.ready_id}."
 
     def reset(
-        self, id: Optional[Union[int, List[int], np.ndarray]] = None
-    ) -> np.ndarray:
+        self,
+        id: Optional[Union[int, List[int], np.ndarray]] = None,
+        **kwargs: Any,
+    ) -> Union[np.ndarray, Tuple[np.ndarray, List[dict]]]:
         """Reset the state of some envs and return initial observations.
 
         If id is None, reset the state of all the environments and return
@@ -209,22 +196,40 @@ class BaseVectorEnv(gym.Env):
         id = self._wrap_id(id)
         if self.is_async:
             self._assert_id(id)
+
         # send(None) == reset() in worker
         for i in id:
-            self.workers[i].send(None)
-        obs_list = [self.workers[i].recv() for i in id]
+            self.workers[i].send(None, **kwargs)
+        ret_list = [self.workers[i].recv() for i in id]
+
+        reset_returns_info = isinstance(ret_list[0], (tuple, list)) and len(
+            ret_list[0]
+        ) == 2 and isinstance(ret_list[0][1], dict)
+        if reset_returns_info:
+            obs_list = [r[0] for r in ret_list]
+        else:
+            obs_list = ret_list
+
+        if isinstance(obs_list[0], tuple):
+            raise TypeError(
+                "Tuple observation space is not supported. ",
+                "Please change it to array or dict space",
+            )
         try:
             obs = np.stack(obs_list)
         except ValueError:  # different len(obs)
             obs = np.array(obs_list, dtype=object)
-        if self.obs_rms and self.update_obs_rms:
-            self.obs_rms.update(obs)
-        return self.normalize_obs(obs)
+
+        if reset_returns_info:
+            infos = [r[1] for r in ret_list]
+            return obs, infos  # type: ignore
+        else:
+            return obs
 
     def step(
         self,
         action: np.ndarray,
-        id: Optional[Union[int, List[int], np.ndarray]] = None
+        id: Optional[Union[int, List[int], np.ndarray]] = None,
     ) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
         """Run one timestep of some environments' dynamics.
 
@@ -264,7 +269,7 @@ class BaseVectorEnv(gym.Env):
                 self.workers[j].send(action[i])
             result = []
             for j in id:
-                obs, rew, done, info = self.workers[j].recv()
+                obs, rew, done, info = self.workers[j].recv()  # type: ignore
                 info["env_id"] = j
                 result.append((obs, rew, done, info))
         else:
@@ -286,7 +291,7 @@ class BaseVectorEnv(gym.Env):
                 waiting_index = self.waiting_conn.index(conn)
                 self.waiting_conn.pop(waiting_index)
                 env_id = self.waiting_id.pop(waiting_index)
-                obs, rew, done, info = conn.recv()
+                obs, rew, done, info = conn.recv()  # type: ignore
                 info["env_id"] = env_id
                 result.append((obs, rew, done, info))
                 self.ready_id.append(env_id)
@@ -298,13 +303,11 @@ class BaseVectorEnv(gym.Env):
         rew_stack, done_stack, info_stack = map(
             np.stack, [rew_list, done_list, info_list]
         )
-        if self.obs_rms and self.update_obs_rms:
-            self.obs_rms.update(obs_stack)
-        return self.normalize_obs(obs_stack), rew_stack, done_stack, info_stack
+        return obs_stack, rew_stack, done_stack, info_stack
 
     def seed(
         self,
-        seed: Optional[Union[int, List[int]]] = None
+        seed: Optional[Union[int, List[int]]] = None,
     ) -> List[Optional[List[int]]]:
         """Set the seed for all environments.
 
@@ -346,15 +349,6 @@ class BaseVectorEnv(gym.Env):
             w.close()
         self.is_closed = True
 
-    def normalize_obs(self, obs: np.ndarray) -> np.ndarray:
-        """Normalize observations by statistics in obs_rms."""
-        if self.obs_rms and self.norm_obs:
-            clip_max = 10.0  # this magic number is from openai baselines
-            # see baselines/common/vec_env/vec_normalize.py#L10
-            obs = (obs - self.obs_rms.mean) / np.sqrt(self.obs_rms.var + self.__eps)
-            obs = np.clip(obs, -clip_max, clip_max)
-        return obs
-
 
 class DummyVectorEnv(BaseVectorEnv):
     """Dummy vectorized environment wrapper, implemented in for-loop.
