@@ -24,7 +24,9 @@ class RayEnvWorker(EnvWorker):
     """Ray worker used in RayVectorEnv."""
 
     def __init__(self, env_fn: Callable[[], gym.Env]) -> None:
-        self.env = ray.remote(_SetAttrWrapper).options(num_cpus=0).remote(env_fn())
+        self.env = ray.remote(_SetAttrWrapper).options(  # type: ignore
+            num_cpus=0
+        ).remote(env_fn())
         super().__init__(env_fn)
 
     def get_env_attr(self, key: str) -> Any:
@@ -33,8 +35,10 @@ class RayEnvWorker(EnvWorker):
     def set_env_attr(self, key: str, value: Any) -> None:
         ray.get(self.env.set_env_attr.remote(key, value))
 
-    def reset(self) -> Any:
-        return ray.get(self.env.reset.remote())
+    def reset(self, **kwargs: Any) -> Any:
+        if "seed" in kwargs:
+            super().seed(kwargs["seed"])
+        return ray.get(self.env.reset.remote(**kwargs))
 
     @staticmethod
     def wait(  # type: ignore
@@ -44,21 +48,25 @@ class RayEnvWorker(EnvWorker):
         ready_results, _ = ray.wait(results, num_returns=wait_num, timeout=timeout)
         return [workers[results.index(result)] for result in ready_results]
 
-    def send(self, action: Optional[np.ndarray]) -> None:
-        # self.action is actually a handle
+    def send(self, action: Optional[np.ndarray], **kwargs: Any) -> None:
+        # self.result is actually a handle
         if action is None:
-            self.result = self.env.reset.remote()
+            self.result = self.env.reset.remote(**kwargs)
         else:
             self.result = self.env.step.remote(action)
 
     def recv(
         self
     ) -> Union[Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray], np.ndarray]:
-        return ray.get(self.result)
+        return ray.get(self.result)  # type: ignore
 
-    def seed(self, seed: Optional[int] = None) -> List[int]:
+    def seed(self, seed: Optional[int] = None) -> Optional[List[int]]:
         super().seed(seed)
-        return ray.get(self.env.seed.remote(seed))
+        try:
+            return ray.get(self.env.seed.remote(seed))
+        except NotImplementedError:
+            self.env.reset.remote(seed=seed)
+            return None
 
     def render(self, **kwargs: Any) -> Any:
         return ray.get(self.env.render.remote(**kwargs))
